/* @(#) implementation of network operations for udpxy
 *
 * Copyright 2008 Pavel V. Cherenkov
 *
 *  This file is part of udpxy.
 *
 *  udpxy is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  udpxy is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with udpxy.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <sys/types.h>
#include <sys/socket.h>
#include <arpa/inet.h>
#include <netinet/in.h>

#include <assert.h>
#include <string.h>
#include <unistd.h>
#include <time.h>
#include <errno.h>
#include <stdio.h>

#include "udpxy.h"
#include "netop.h"
#include "util.h"
#include "mtrace.h"


extern FILE* g_flog;    /* application log */


/* set up (server) listening sockfd
 */
int
setup_listener( const char* ipaddr, int port, int* sockfd )
{
    int rc, lsock;
    struct sockaddr_in servaddr;
    const int ON = 1;

    extern const char IPv4_ALL[];

    assert( (port > 0) && sockfd && ipaddr );
    (void)IPv4_ALL;
    TRACE( (void)tmfprintf( g_flog, "Setting up listener for [%s:%d]\n",
                ipaddr[0] ? ipaddr : IPv4_ALL, port) );

    rc = ERR_INTERNAL;
    do {
        lsock = socket( AF_INET, SOCK_STREAM, 0 );
        if( -1 == lsock ) break;

        (void) memset( &servaddr, 0, sizeof(servaddr) );
        servaddr.sin_family = AF_INET;
        servaddr.sin_port = htons( (short)port );

        if( '\0' != ipaddr[0] ) {
            if( 1 != inet_aton(ipaddr, &servaddr.sin_addr) ) {
                TRACE( (void)tmfprintf( g_flog, "Invalid server IP: [%s]\n",
                        ipaddr) );

                rc = ERR_PARAM;
                break;
            }
        }
        else {
            servaddr.sin_addr.s_addr = htonl(INADDR_ANY);
        }

        rc = setsockopt( lsock, SOL_SOCKET, SO_REUSEADDR,
                         &ON, sizeof(ON) );
        if( 0 != rc ) {
            mperror(g_flog, errno, "%s: setsockopt SO_REUSEADDR",
                    __func__);
            break;
        }

        rc = bind( lsock, (struct sockaddr*)&servaddr, sizeof(servaddr) );
        if( 0 != rc ) break;

        rc = listen( lsock, LQ_BACKLOG );
        if( 0 != rc ) break;

        rc = 0;
    } while(0);

    if( 0 != rc ) {
        if(errno)
            mperror(g_flog, errno, "%s: socket/bind/listen error",
                    __func__);
        if( lsock ) {
            (void) close( lsock );
        }
    }
    else {
        *sockfd = lsock;
        TRACE( (void)tmfprintf( g_flog, "Server socket=[%d] is set up\n",
                lsock) );
    }

    return rc;
}


/* set up the socket to receive multicast data
 *
 */
int
setup_mcast_listener( struct sockaddr_in*   sa,
                      const struct in_addr* mifaddr,
                      int*                  mcastfd,
                      int                   sockbuflen )
{
    int sockfd, rc;
    int ON = 1;
    struct ip_mreq mreq;
    int buflen = sockbuflen;

    assert( sa && mifaddr && mcastfd );

    TRACE( (void)tmfprintf( g_flog, "Setting up multicast listener\n") );
    rc = ERR_INTERNAL;
    do {
        sockfd = socket( AF_INET, SOCK_DGRAM, 0 );
        if( -1 == sockfd ) {
            mperror(g_flog, errno, "%s: socket", __func__);
            break;
        }

        if( buflen != 0 ) {
            rc = setsockopt( sockfd, SOL_SOCKET, SO_RCVBUF,
                             &buflen, sizeof(buflen) );
            if( 0 != rc ) {
                mperror(g_flog, errno, "%s: setsockopt SO_RCVBUF",
                        __func__);
                break;
            }
            TRACE( (void)tmfprintf( g_flog,
                    "multicast buffer size set to [%d] bytes\n",
                    buflen ) );
        }


        rc = setsockopt( sockfd, SOL_SOCKET, SO_REUSEADDR,
                         &ON, sizeof(ON) );
        if( 0 != rc ) {
            mperror(g_flog, errno, "%s: setsockopt SO_REUSEADDR",
                    __func__);
            break;
        }

        rc = bind( sockfd, (struct sockaddr*)sa, sizeof(*sa) );
        if( 0 != rc ) {
            mperror(g_flog, errno, "%s: bind", __func__);
            break;
        }

        (void) memset( &mreq, 0, sizeof(mreq) );
        (void) memcpy( &mreq.imr_multiaddr, &(sa->sin_addr),
                sizeof(struct in_addr) );

        (void) memcpy( &mreq.imr_interface, mifaddr,
                sizeof(struct in_addr) );

        rc = setsockopt( sockfd, IPPROTO_IP, IP_ADD_MEMBERSHIP,
                &mreq, sizeof(mreq) );
        if( 0 != rc ) {
            mperror(g_flog, errno, "%s: setsockopt IP_ADD_MEMBERSHIP",
                    __func__);
            break;
        }
    } while(0);

    if( 0 == rc ) {
        *mcastfd = sockfd;
        TRACE( (void)tmfprintf( g_flog, "Mcast listener socket=[%d] set up\n",
                                sockfd) );
    }
    else {
        (void)close(sockfd);
    }

    return rc;
}


/* unsubscribe from multicast and close the reader socket
 *
 */
void
close_mcast_listener( int msockfd, const struct in_addr* mifaddr )
{
    struct ip_mreq mreq;

    assert( mifaddr );

    if( msockfd <= 0 ) return;

    (void) memset( &mreq, 0, sizeof(mreq) );
    (void) memcpy( &mreq.imr_interface, mifaddr,
                sizeof(struct in_addr) );

    (void) setsockopt( msockfd, IPPROTO_IP, IP_DROP_MEMBERSHIP,
                    &mreq, sizeof(mreq) );
    (void) close( msockfd );

    TRACE( (void)tmfprintf( g_flog, "Mcast listener socket=[%d] closed\n",
                msockfd) );
    return;
}


/* set send/receive timeouts on socket(s)
 */
int
set_timeouts( int rsock, int ssock, u_short rcv_tmout, u_short snd_tmout )
{
    struct timeval rtv, stv;
    int rc = 0;

    rtv.tv_sec = rcv_tmout;
    rtv.tv_usec = 0;
    rc = setsockopt( rsock, SOL_SOCKET, SO_RCVTIMEO, &rtv, sizeof(rtv) );
    if( -1 == rc ) {
        mperror(g_flog, errno, "%s: setsockopt - SO_RCVTIMEO",
                __func__);
        return ERR_INTERNAL;
    }

    stv.tv_sec = snd_tmout;
    stv.tv_usec = 0;
    rc = setsockopt( ssock, SOL_SOCKET, SO_SNDTIMEO, &stv, sizeof(stv) );
    if( -1 == rc ) {
        mperror(g_flog, errno, "%s: setsockopt - SO_SNDTIMEO", __func__);
        return ERR_INTERNAL;
    }

    return rc;
}


/* set socket's send buffer value
 */
int
set_sendbuf( int sockfd, const size_t len )
{
    size_t data_len = len;
    int rc = 0;

    rc = setsockopt( sockfd, SOL_SOCKET, SO_SNDBUF,
                    &data_len, sizeof(data_len) );
    if( 0 != rc ) {
        mperror(g_flog, errno, "%s: setsockopt SO_SNDBUF", __func__);
        return -1;
    }
    else {
        TRACE( (void)tmfprintf( g_flog,
            "send buffer size set to [%u] bytes for socket [%d]\n",
            data_len, sockfd ) );
    }

    return rc;
}


/* __EOF__ */

